using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace Tutorial_4_5
{
    class GrfxWindowXNA
    {
        //XNA graphics device
        private GraphicsDevice m_XNA_GraphicsDevice = null;
        private BasicEffect m_BasicEffect;

        private Circle2D m_SmallCircle;
        private Rectangle2D m_LargeSquare;

        public GrfxWindowXNA()
        {
            m_SmallCircle = new Circle2D();
            m_LargeSquare = new Rectangle2D();

            m_LargeSquare.SetCenter(120, 82);
            m_LargeSquare.SetSize(80, 80);

            m_SmallCircle.SetCenter(185, 147);
            m_SmallCircle.SetRadius(20);
        }

        //Replaces the PlaceHolder with an XNA graphics the GrfxWindowXNA 
        public void ReplaceFormControl(Control PlaceHolder)
        {
            Create_Graphics_Device(PlaceHolder.Handle, PlaceHolder.Width, PlaceHolder.Height);

            //Links the PlaceHolder's Paint Event with the GrfxWindowXNA's OnPaint event
            PlaceHolder.Paint += new PaintEventHandler(OnPaint);
        }

        //Clears the graphics device and draws the two rectangles to it
        private void OnPaint(object sender, EventArgs e)
        {
            //set the model matrix to identity
            m_BasicEffect.World = Matrix.Identity;
            m_BasicEffect.View = Matrix.Identity;
            m_BasicEffect.Projection = Matrix.Identity;
            
            //taking parameters from the grfxPanel guarentees that you get the correct
            //aspect ratio for graphics rendering
            Panel grfxPanel = (Panel)sender;

            Matrix World2NDC = new Matrix();
            Vector3 scale = new Vector3(2f / grfxPanel.Width, 2f / grfxPanel.Height, 1f);
            Vector3 translate = new Vector3(-1f, -1f, 0f);
            World2NDC = Matrix.CreateScale(scale) * Matrix.CreateTranslation(translate);

            m_BasicEffect.View = World2NDC;
            m_BasicEffect.VertexColorEnabled = true;

            
            //clear the graphics device with a grey background color
            m_XNA_GraphicsDevice.Clear(new Color(new Vector4(.8f, .8f, .8f, 0f)));

           // This says "pass.Begin()"
            m_BasicEffect.CurrentTechnique.Passes[0].Apply();

            //large square
            m_LargeSquare.Draw(m_XNA_GraphicsDevice);

            //small square
            m_SmallCircle.Draw(m_XNA_GraphicsDevice);


            //Show what we drew
            m_XNA_GraphicsDevice.Present();
        }

        //Creates the XNA graphics device if one doesn't already exist
        private void Create_Graphics_Device(IntPtr handle, int width, int height)
        {
            if (m_XNA_GraphicsDevice == null) //don't re-create the graphics device if it already exists
            {
                //define presentation parameters
                PresentationParameters pp = new PresentationParameters();
                pp.BackBufferFormat = SurfaceFormat.Color;
                pp.BackBufferWidth = width;
                pp.BackBufferHeight = height;
                pp.DeviceWindowHandle = handle;
                pp.IsFullScreen = false;

                //create an XNA graphics device
                m_XNA_GraphicsDevice = new GraphicsDevice(GraphicsAdapter.DefaultAdapter, 
                    GraphicsProfile.Reach, // allow support on as many devices as possible 
                    pp);

                //
                RasterizerState st = new RasterizerState();
                st.CullMode = CullMode.None;
                st.FillMode = FillMode.Solid;
                m_XNA_GraphicsDevice.RasterizerState = st;

                //check if graphics device was created
                Debug.Assert(m_XNA_GraphicsDevice != null, "XNA Graphics Device did not Initialize");

                m_BasicEffect = new BasicEffect(m_XNA_GraphicsDevice);
            }
        }
    }
}
