using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib10
{
    public class UWB_XNADrawHelper : UWB_DrawHelper
    {
		UWB_XNAD3DMatrixStack mMatrixStack;

        public UWB_XNADrawHelper()
        {
			mMatrixStack = new UWB_XNAD3DMatrixStack();
        }

        public override bool drawPoint(Vector3 position)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawPoint call");

            // Draw points as rectangles
            VertexPositionColor[] v = new VertexPositionColor[4];
            float offset = mPointSize / 2f;

            for (int i = 0; i < 4; i++)
            {
                v[i] = new VertexPositionColor();
                v[i].Color = mColor1;
            }

            v[0].Position = new Vector3(position.X - offset, position.Y + offset, 0);
            v[1].Position = new Vector3(position.X - offset, position.Y - offset, 0);
            v[2].Position = new Vector3(position.X + offset, position.Y + offset, 0);
            v[3].Position = new Vector3(position.X + offset, position.Y - offset, 0);

            if (device.RasterizerState.FillMode != FillMode.Solid)
            {
                RasterizerState s = new RasterizerState();
                s.CullMode = CullMode.None;
                s.FillMode = FillMode.Solid;
                device.RasterizerState = s;
            }

            device.DrawUserPrimitives(PrimitiveType.TriangleStrip, v, 0, 2);
            return true;
        }

        public override bool drawLine(Vector3 start, Vector3 end)
        {
			GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
			Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");

            VertexPositionColor[] v = new VertexPositionColor[2];

			Color flatColor = mColor1;

			v[0].Position = start;
			v[0].Color = flatColor;
			v[1].Position = end;
			if (mShadeMode == eShadeMode.smFlat)
				v[1].Color = flatColor;
			else
				v[1].Color = mColor2;

			device.DrawUserPrimitives(PrimitiveType.LineList, v, 0, 1);

            return true;
        }

        public override bool drawCircle(Vector3 center, float radius)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");
            if (radius > 0)
            {
                int kNumTriangles = 40;
                if (mLod == eLevelofDetail.lodLow)
                    kNumTriangles = 10;
                else if (mLod == eLevelofDetail.lodMed)
                    kNumTriangles = 20;


                VertexPositionColor[] v = new VertexPositionColor[kNumTriangles * 3];

                float theta = (2.0f * (float)Math.PI) / kNumTriangles;

                v[0].Position = center;
                if (mShadeMode == eShadeMode.smGouraud)
                    v[0].Color = mColor2;
                else
                    v[0].Color = mColor1;

                float px = center.X + radius;
                float py = center.Y;

                for (int i = 0; i < kNumTriangles; i++)
                {
                    int offset = i * 3;
                    float x = center.X + (radius * (float)Math.Cos((i + 1) * theta));
                    float y = center.Y + (radius * (float)Math.Sin((i + 1) * theta));
                    v[offset].Position = v[0].Position;
                    v[offset].Color = v[0].Color;
                    v[offset + 1].Position = new Vector3(px, py, 0f);
                    v[offset + 1].Color = mColor1;
                    v[offset + 2].Position = new Vector3(x, y, 0f);
                    v[offset + 2].Color = mColor1;

                    px = x;
                    py = y;
                }

                device.DrawUserPrimitives(PrimitiveType.TriangleList, v, 0, kNumTriangles);
            }
            return true;
        }

		public override bool drawRectangle(Vector3 min, Vector3 max)
		{
			GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
			Debug.Assert(device != null, "device not valid on DrawHelper drawRectangle call");

			VertexPositionColor[] v = new VertexPositionColor[12];
			Color flatColor = mColor1;

            if (mShadeMode == eShadeMode.smGouraud)
                v[0].Color = mColor2;
            else
                v[0].Color = flatColor;

            v[0].Position = Vector3.Divide((min + max), 2);
			v[1].Position = new Vector3(max.X, min.Y, min.Z);
			v[1].Color = flatColor;
			v[2].Position = max;
            v[2].Color = flatColor;

            v[3] = v[0];
            v[4] = v[2];
            v[5].Position = new Vector3(min.X, max.Y, max.Z);
            v[5].Color = flatColor;

            v[6] = v[0];
            v[7] = v[5];
            v[8].Position = min;
            v[8].Color = flatColor;

            v[9] = v[0];
            v[10] = v[8];
            v[11] = v[1];

            device.DrawUserPrimitives(PrimitiveType.TriangleList, v, 0, 4);
            return true;
		}

        public override void setFillMode(eFillMode fillMode)
        {
            GraphicsDevice graphicsDevice = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            bool needToSetMode = false;
            FillMode newMode = FillMode.Solid;
            switch (fillMode)
            {
                case eFillMode.fmWireframe:
                case eFillMode.fmPoint:
                    needToSetMode = (graphicsDevice.RasterizerState.FillMode != FillMode.WireFrame);
                    newMode = FillMode.WireFrame;
                    break;
                case eFillMode.fmSolid:
                    needToSetMode = (graphicsDevice.RasterizerState.FillMode != FillMode.Solid);
                    break;
            }
            RasterizerState s = new RasterizerState();
            s.CullMode = CullMode.None;
            s.FillMode = newMode;
            graphicsDevice.RasterizerState = s;
        }

		public override bool accumulateModelTransform(
			Vector3 translation,
			Vector3 scale,
			float rotationRadians,
			Vector3 pivot)
		{
			if (UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice() == null)
				return false;

			Vector3 z_axis = new Vector3(0, 0, 1);
			mMatrixStack.TranslateLocal(translation);
			mMatrixStack.TranslateLocal(pivot);
			mMatrixStack.RotateAxisLocal(z_axis, rotationRadians);
			mMatrixStack.ScaleLocal(scale);
			mMatrixStack.TranslateLocal(-pivot);

			UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(mMatrixStack.Top());
			return true;

		}

		public override bool pushModelTransform()
		{
			if (UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice() == null)
				return false;

			mMatrixStack.Push();
			UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(mMatrixStack.Top());
			return true;
		}

		public override bool popModelTransform()
		{
			if (UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice() == null)
				return false;

			mMatrixStack.Pop();
			UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(mMatrixStack.Top());
			return true;
		}

		public override bool initializeModelTransform()
		{
			if (UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice() == null)
				return false;

			mMatrixStack.LoadIdentity();
			UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(mMatrixStack.Top());
			return true;
		}
    }
}
