using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace UWBGL_XNA_Lib11
{
	public class UWB_SceneNode
	{
		private string mName;
		bool mPivotVisible;
		UWB_XFormInfo mXFormInfo;
		UWB_Array<UWB_SceneNode> mChildNodes;
		UWB_Primitive mPrimitive;

		public UWB_SceneNode()
		{
			mPrimitive = new UWB_Primitive();
			mName = "SceneNode";
			mPivotVisible = false;

			mXFormInfo = new UWB_XFormInfo();
			mChildNodes = new UWB_Array<UWB_SceneNode>();
		}

		public void draw(eLevelofDetail lod, UWB_DrawHelper drawHelper)
		{
			drawHelper.pushModelTransform();
			mXFormInfo.setupModelStack(drawHelper);

			if (mPrimitive != null)
			{
				mPrimitive.draw(lod, drawHelper);
			}

			int count = mChildNodes.count();
			for (int i = 0; i < count; i++)
			{
				mChildNodes.getItem(i).draw(lod, drawHelper);
			}

			if (mPivotVisible)
			{
				mXFormInfo.drawPivot(drawHelper, .2f);
			}

			drawHelper.popModelTransform();
		}

		public void setPrimitive(UWB_Primitive primitive)
		{
			mPrimitive = primitive;
		}

		public UWB_Primitive getPrimitive()
		{
			return mPrimitive;
		}

		public void insertChildNode(UWB_SceneNode childNode)
		{
			mChildNodes.append(childNode);
		}

		public int numChildren()
		{
			return mChildNodes.count();
		}

		public UWB_SceneNode getChildNode(int index)
		{
			if (index < 0 || index >= numChildren())
			{
				return null;
			}

			return mChildNodes.getItem(index);
		}

		public UWB_XFormInfo getXFormInfo()
		{
			return mXFormInfo;
		}

		public void setXFormInfo(UWB_XFormInfo xForm)
		{
			mXFormInfo = xForm;
		}

		public void setPivotVisible(bool visible)
		{
			mPivotVisible = visible;
		}

		public bool isPivotVisible()
		{
			return mPivotVisible;
		}
	}
}
