using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib11;

namespace XNA_SceneNode
{
	public class Model
	{
		private UWB_SceneNode mScene;
		private bool mParentActive;
		private bool mShowPivot;

		private UWB_Clock mStopWatch;
		private UWB_BoundingBox mWorldBounds;
		private UWB_XNADrawHelper mDrawHelper;

		const int kBodyGrayColor = 195;

		public Model()
		{
			mParentActive = true;
			mShowPivot = false;

			mScene = new UWB_SceneNode();

			UWB_PrimitiveRectangle rectangle = new UWB_PrimitiveRectangle();
			rectangle.setCorners(new Vector3(0, -1, 0), new Vector3(3, 1, 0));
			rectangle.setFlatColor(new Color(150, 0, 0));
			rectangle.setShadeMode(eShadeMode.smFlat);

			mScene.setPrimitive(rectangle);

			UWB_SceneNode childNode = new UWB_SceneNode();
			UWB_PrimitiveCircle circle = new UWB_PrimitiveCircle();
			circle.setCenter(new Vector3(4, 0, 0));
			circle.setRadius(1f);
			circle.setFlatColor(new Color(0, 150, 0));
			circle.setShadeMode(eShadeMode.smFlat);
			childNode.setPrimitive(circle);

			UWB_XFormInfo xForm = new UWB_XFormInfo();
			xForm.setPivot(new Vector3(3, 0, 0));
			childNode.setXFormInfo(xForm);
			mScene.insertChildNode(childNode);
			
			mDrawHelper = new UWB_XNADrawHelper();
			mStopWatch = new UWB_Clock();
			mWorldBounds = new UWB_BoundingBox(new Vector3(-10, -10, 0), new Vector3(10, 10, 0));
		}

		public UWB_XFormInfo getActiveTransform()
		{
			if (mParentActive)
				return mScene.getXFormInfo();

			UWB_SceneNode childNode = mScene.getChildNode(0);
			if (childNode != null)
			{
				return childNode.getXFormInfo();
			}

			return new UWB_XFormInfo();
		}

		public void setActiveTransform(UWB_XFormInfo xForm)
		{
			if (mParentActive)
				mScene.setXFormInfo(xForm);
			else
			{
				UWB_SceneNode childNode = mScene.getChildNode(0);
				if (childNode != null)
					childNode.setXFormInfo(xForm);
			}
		}

		public void activateChildTransform()
		{
			mParentActive = false;
			if (mShowPivot)
			{
				mScene.setPivotVisible(false);
				mScene.getChildNode(0).setPivotVisible(true);
			}
		}

		public void activateParentTransform()
		{
			mParentActive = true;
			if (mShowPivot)
			{
				mScene.setPivotVisible(true);
				mScene.getChildNode(0).setPivotVisible(false);
			}
		}

		public void setShowPivot(bool show)
		{
			mShowPivot = show;
			mScene.setPivotVisible(mShowPivot && mParentActive);
			mScene.getChildNode(0).setPivotVisible(mShowPivot && !mParentActive);
		}

		private bool getShowPivot()
		{
			return mShowPivot;
		}

		public void updateSimulation()
		{
			//nothing to do here
		}

		public void getWorldBounds(ref float width, ref float height)
		{
			width = mWorldBounds.width();
			height = mWorldBounds.height();
		}

		public UWB_BoundingBox getWorldBounds()
		{
			return mWorldBounds;
		}

		public void drawGrid(UWB_DrawHelper drawHelper, UWB_BoundingBox bounds)
		{
			Color gridColor = new Color(150, 150, 150);
			Color axisColor = new Color(130, 30, 30);

			drawHelper.setColor1(gridColor);
			drawHelper.setColor1(gridColor);

			Vector3 minPoint = bounds.getMin();
			Vector3 maxPoint = bounds.getMax();

			float y = minPoint.Y;
			while (y < maxPoint.Y)
			{
				if (y == 0)
				{
					drawHelper.setColor1(axisColor);
					drawHelper.setColor2(axisColor);
					drawHelper.drawLine(new Vector3(minPoint.X, y, 0), new Vector3(maxPoint.X, y, 0));
					drawHelper.setColor1(gridColor);
					drawHelper.setColor2(gridColor);
				}
				else
				{
					drawHelper.setColor1(gridColor);
					drawHelper.setColor2(gridColor);
					drawHelper.drawLine(new Vector3(minPoint.X, y, 0), new Vector3(maxPoint.X, y, 0));
				}
				y += 1;
			}

			float x = minPoint.X;
			while (x < maxPoint.X)
			{
				if (x == 0)
				{
					drawHelper.setColor1(axisColor);
					drawHelper.setColor2(axisColor);
					drawHelper.drawLine(new Vector3(x, minPoint.Y, 0), new Vector3(x, maxPoint.Y, 0));
					drawHelper.setColor1(gridColor);
					drawHelper.setColor2(gridColor);
				}
				else
				{
					drawHelper.setColor1(gridColor);
					drawHelper.setColor2(gridColor);
					drawHelper.drawLine(new Vector3(x, minPoint.Y, 0), new Vector3(x, maxPoint.Y, 0));
				}
				x += 1;
			}
		}

		public void drawModel()
		{
			mDrawHelper.initializeModelTransform();

			drawGrid(mDrawHelper, mWorldBounds);

			mDrawHelper.pushModelTransform();

			eLevelofDetail lod = mDrawHelper.getLod();
			mScene.draw(lod, mDrawHelper);
			
			mDrawHelper.popModelTransform();
		}
	}
}

