using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib13
{
    public class RGraphicsDeviceService : IGraphicsDeviceService
    {
        private GraphicsDevice m_graphicsDevice;

        public event EventHandler<EventArgs> DeviceCreated;
        public event EventHandler<EventArgs> DeviceDisposing;
        public event EventHandler<EventArgs> DeviceReset;
        public event EventHandler<EventArgs> DeviceResetting;

        public GraphicsDevice GraphicsDevice
        {
            get { return this.m_graphicsDevice; }
            set { m_graphicsDevice = value; }
        } 
    }

    public class RGameView : IServiceProvider
    {
        public RGraphicsDeviceService m_graphicsDeviceService;

        public void SetService(RGraphicsDeviceService s)
        {
            m_graphicsDeviceService = s;
        }

        public object GetService(System.Type serviceType)
        {
            return this.m_graphicsDeviceService;
        }
    }
    
    public class UWB_XNAGraphicsSystem
    {
        private BasicEffect m_BasicEffect;
        private GraphicsDevice m_XNA_GraphicsDevice; 
        private RasterizerState m_DefaultState;
        
        private ContentManager mResources;
        public static UWB_XNAGraphicsSystem m_TheAPI = new UWB_XNAGraphicsSystem();
        private RGameView mService;

        public BasicEffect MyEffect
        {
            get
            {
                return m_BasicEffect;
            }
            set
            {
                m_BasicEffect = value;
            }
        }

        private UWB_XNAGraphicsSystem()
        {
        }

        public bool CreateGraphicsContext(IntPtr hwnd, ref PresentationParameters pp)
        {
            //define presentation parameters
            pp = new PresentationParameters();
            pp.BackBufferFormat = SurfaceFormat.Color;
            pp.DeviceWindowHandle = hwnd;
            pp.IsFullScreen = false;
            pp.BackBufferHeight = Control.FromHandle(hwnd).Height;
            pp.BackBufferWidth = Control.FromHandle(hwnd).Width;

            if (m_XNA_GraphicsDevice != null)
                return false;
            //create an XNA graphics device
            m_XNA_GraphicsDevice = new GraphicsDevice(GraphicsAdapter.DefaultAdapter,
                GraphicsProfile.HiDef, // allow support on as many devices as possible 
                pp);

            //check if graphics device was created
            Debug.Assert(m_XNA_GraphicsDevice != null, "XNA Graphics Device did not Initialize");

            //
            m_DefaultState = new RasterizerState();
            m_DefaultState.CullMode = CullMode.None;
            m_DefaultState.FillMode = FillMode.Solid;
            m_XNA_GraphicsDevice.RasterizerState = m_DefaultState;

            //check if graphics device was created
            Debug.Assert(m_XNA_GraphicsDevice != null, "XNA Graphics Device did not Initialize");

            m_BasicEffect = new BasicEffect(m_XNA_GraphicsDevice);

            //set the model matrix to identity
            m_BasicEffect.World = Matrix.Identity;
            m_BasicEffect.View = Matrix.Identity;
            m_BasicEffect.Projection = Matrix.Identity;

            RGraphicsDeviceService gS = new RGraphicsDeviceService();
            gS.GraphicsDevice = m_XNA_GraphicsDevice;
            mService = new RGameView();
            mService.SetService(gS);
            mResources = new ContentManager(mService);
            mResources.RootDirectory = ".";

            return true;
        }

        public GraphicsDevice GetActiveDevice()
        {
            return m_XNA_GraphicsDevice;
        }


        public bool BeginScene(PresentationParameters pp)
        {
            if (m_XNA_GraphicsDevice == null)
                return false;
            m_XNA_GraphicsDevice.Reset(pp);
            m_XNA_GraphicsDevice.RasterizerState = m_DefaultState;
            m_XNA_GraphicsDevice.Clear(new Color(new Vector4(.8f, .8f, .8f, 1f)));

            m_BasicEffect.VertexColorEnabled = true;

            m_BasicEffect.CurrentTechnique.Passes[0].Apply();

            return true;
        }

        public bool EndSceneAndShow(PresentationParameters pp)
        {
            if (m_XNA_GraphicsDevice == null)
                return false;

            m_XNA_GraphicsDevice.Present();

            return true;
        }

        public void setViewMatrix(Matrix matrix)
        {
            m_BasicEffect.View = matrix;
            m_BasicEffect.CurrentTechnique.Passes[0].Apply();
        }

        public void setWorldMatrix(Matrix matrix)
        {
            m_BasicEffect.World = matrix;
            m_BasicEffect.CurrentTechnique.Passes[0].Apply();
        }

        public bool ActivateTexture(string texName)
        {
            Texture2D t = null;
            
            try
            {
                // check Textures folder, current folder, and what else?
                t = mResources.Load<Texture2D>("Content\\Textures\\" + texName);
                m_BasicEffect.Texture = t;
                m_BasicEffect.TextureEnabled = true;
                m_BasicEffect.VertexColorEnabled = false;
            }
            catch (Exception e)
            {
                //texture load failed - disable texturing
                m_BasicEffect.Texture = null;
                m_BasicEffect.TextureEnabled = false;
                m_BasicEffect.VertexColorEnabled = true;
                UWB_Utility.echoToStatusArea((e.Message));
            }
            m_BasicEffect.CurrentTechnique.Passes[0].Apply();
            
            return true;
        }

        public void DeactivateTexture()
        {
            m_BasicEffect.TextureEnabled = false;
            m_BasicEffect.Texture = null;
            m_BasicEffect.VertexColorEnabled = true;
            m_BasicEffect.CurrentTechnique.Passes[0].Apply();
        }
    }
}