using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib1
{

    public class UWB_XNAGraphicsSystem
    {
        private GraphicsDevice m_XNA_GraphicsDevice;
        private BasicEffect m_BasicEffect;

        public static UWB_XNAGraphicsSystem m_TheAPI = new UWB_XNAGraphicsSystem();

        public bool CreateGraphicsContext(IntPtr hwnd)
        {
            if (m_XNA_GraphicsDevice != null)
            {
                return false;
            }

            //define presentation parameters
            PresentationParameters pp = new PresentationParameters();
            pp.BackBufferFormat = SurfaceFormat.Color;
            pp.DeviceWindowHandle = hwnd;
            pp.IsFullScreen = false;
            pp.BackBufferHeight = Control.FromHandle(hwnd).Height;
            pp.BackBufferWidth = Control.FromHandle(hwnd).Width;

            //create an XNA graphics device
            m_XNA_GraphicsDevice = new GraphicsDevice(GraphicsAdapter.DefaultAdapter,
                GraphicsProfile.Reach, // allow support on as many devices as possible 
                pp);

            //check if graphics device was created
            Debug.Assert(m_XNA_GraphicsDevice != null, "XNA Graphics Device did not Initialize");

            //
            RasterizerState st = new RasterizerState();
            st.CullMode = CullMode.None;
            st.FillMode = FillMode.Solid;
            m_XNA_GraphicsDevice.RasterizerState = st;

            //check if graphics device was created
            Debug.Assert(m_XNA_GraphicsDevice != null, "XNA Graphics Device did not Initialize");

            m_BasicEffect = new BasicEffect(m_XNA_GraphicsDevice);

            //set the model matrix to identity
            m_BasicEffect.World = Matrix.Identity;
            m_BasicEffect.View = Matrix.Identity;
            m_BasicEffect.Projection = Matrix.Identity;

            return true;
        }

        public GraphicsDevice GetActiveDevice()
        {
            return m_XNA_GraphicsDevice;
        }

        public bool BeginScene()
        {
            if (m_XNA_GraphicsDevice == null)
                return false;

            m_XNA_GraphicsDevice.Clear(new Color(new Vector4(.8f, .8f, .8f, 0)));

            System.Drawing.Size windowSize = Control.FromHandle(m_XNA_GraphicsDevice.PresentationParameters.DeviceWindowHandle).Size;
            if (windowSize.Width <= 0 || windowSize.Height <= 0)
                return false;

            Matrix World2NDC = new Matrix();
            Vector3 scale = new Vector3(2f / windowSize.Width, 2f / windowSize.Height, 1f);
            Vector3 translate = new Vector3(-1f, -1f, 0f);
            World2NDC = Matrix.CreateScale(scale) * Matrix.CreateTranslation(translate);

            m_BasicEffect.View = World2NDC;
            m_BasicEffect.VertexColorEnabled = true;

            m_BasicEffect.CurrentTechnique.Passes[0].Apply();

            return true;
        }

        public bool EndSceneAndShow()
        {
            if (m_XNA_GraphicsDevice == null)
                return false;

            m_XNA_GraphicsDevice.Present();
            return true;
        }

    }
}
