using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib2
{
    public class UWB_XNACircleGeometry
    {
        protected Vector3 m_center;
        protected float m_radius;
        protected Color m_fillColor;
        protected bool m_fillMode;

        public UWB_XNACircleGeometry()
        {
            m_center = new Vector3(0, 0, 0);
            m_radius = 0;
            m_fillColor = new Color(new Vector4(.4f, .4f, .4f, 1f));
            m_fillMode = true;
        }

        public void SetCenter(float x, float y)
        {
            m_center.X = x;
            m_center.Y = y;
        }

        public void SetCenter(Vector3 center)
        {
            m_center = center;
        }

        public Vector3 GetCenter()
        {
            return m_center;
        }

        public void SetRadius(float radius)
        {
            m_radius = radius;
        }

        public float GetRadius()
        {
            return m_radius;
        }

        public void Draw()
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
             Debug.Assert(device != null, "device not valid on Circle2D draw call");
             if (m_radius > 0)
             {
                 const int kNumTriangles = 100;
                 VertexPositionColor[] v = new VertexPositionColor[kNumTriangles * 3];

                 float theta = (2.0f * (float)Math.PI) / kNumTriangles;

                 v[0].Position = new Vector3(m_center.X, m_center.Y, 0f);
                 v[0].Color = new Color(new Vector4(0.4f, 0.4f, 0.4f, 1f));
                 float px = m_center.X + m_radius;
                 float py = m_center.Y;

                 for (int i = 0; i < kNumTriangles; i++)
                 {
                     int offset = i * 3;
                     float x = m_center.X + (m_radius * (float)Math.Cos((i + 1) * theta));
                     float y = m_center.Y + (m_radius * (float)Math.Sin((i + 1) * theta));
                     v[offset].Position = v[0].Position;
                     v[offset].Color = v[0].Color;
                     v[offset + 1].Position = new Vector3(px, py, 0f);
                     v[offset + 1].Color = v[0].Color;
                     v[offset + 2].Position = new Vector3(x, y, 0f);
                     v[offset + 2].Color = v[0].Color;

                     px = x;
                     py = y;
                 }

                 device.DrawUserPrimitives(PrimitiveType.TriangleList, v, 0, kNumTriangles);
             }
        }
    }
}
