using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib2;

namespace XNA_SimulateGravity
{
    class DrawAndMouseHandler : UWB_XNAWindowHandler
    {
        private UWBGL_UIWindow mWindow;

        public DrawAndMouseHandler()
        {
            mWindow = new UWBGL_UIWindow(true);
        }

        public bool Initialize(Control PlaceHolder)
        {
            if (!mWindow.ReplaceFormControl(PlaceHolder.Handle))
                return false;

            if (!mWindow.AttachHandler(this))
                return false;
            
            return true;
        }

        public override void DrawGraphics()
        {
            if (mWindow.Width <= 0 || mWindow.Height <= 0)
                return;

            BeginScene();

            Program.getModel().Draw();

            EndSceneAndShow();
        }

        public override void OnMouseButton(bool down, MouseEventArgs e)
        {
            int deviceX = 0, deviceY = 0;
            hardwareToDevice(e.X, e.Y, ref deviceX, ref deviceY);

            string key = "";
            Keys modkey = Control.ModifierKeys;
            if (modkey == Keys.Control)
                key += "[CTRL]";
            if (modkey == Keys.Shift)
                key += "[SHIFT]";
            if (key.Length == 0)
                key += "[NONE]";

            string btn = "";
            switch (e.Button)
            {
                case MouseButtons.Left:
                    btn = "LMB";
                    if (down)
                    {
                        Program.getModel().setHeroBallAnimation(false);
                        Program.getModel().createHeroBall((float)deviceX, (float)deviceY);
                    }
                    else
                        Program.getModel().setHeroBallAnimation(true);
                    break;
                case MouseButtons.Right:
                    btn = "RMB";
                    if(down)
                    {
                        hardwareToDevice(e.X, e.Y, ref deviceX, ref deviceY);
                        Program.getModel().setHeroBallAnimation(false);
                        Program.getModel().moveHeroBallTo((float)deviceX, (float)deviceY);
                    }
                    else
                        Program.getModel().setHeroBallAnimation(true);
                    break;
                case MouseButtons.Middle:
                    btn = "MMB";
                    break;
            }

            string msg = "Viewport: " + btn + ": Position: (" + deviceX.ToString() + " " + deviceY.ToString() + ") Keys: " + key;
            UWB_XNAUtility.echoToStatusArea(msg);
        }

        public override void OnMouseMove(MouseEventArgs e)
        {
            int deviceX = 0, deviceY = 0;
            hardwareToDevice(e.X, e.Y, ref deviceX, ref deviceY);

            string key = "";
            Keys modkey = Control.ModifierKeys;
            if (modkey == Keys.Control)
                key += "[CTRL]";
            if (modkey == Keys.Shift)
                key += "[SHIFT]";
            if (key.Length == 0)
                key += "[NONE]";

            string btn = "";
            switch (e.Button)
            {
                case MouseButtons.Left:
                    btn = "LMB";
                    Program.getModel().setHeroBallAnimation(false);
                    Program.getModel().resizeHeroBall((float)deviceX, (float)deviceY);
                    break;
                case MouseButtons.Right:
                    btn = "RMB";
                    hardwareToDevice(e.X, e.Y, ref deviceX, ref deviceY);
                    Program.getModel().setHeroBallAnimation(false);
                    Program.getModel().moveHeroBallTo((float)deviceX, (float)deviceY);
                    break;
                case MouseButtons.Middle:
                    btn = "MMB";
                    break;
            }

            string msg = "Viewport: " + btn + ": Position: (" + deviceX.ToString() + " " + deviceY.ToString() + ") Keys: " + key;
            UWB_XNAUtility.echoToStatusArea(msg);
        }
    }
}
