using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using UWBGL_XNA_Lib2;
using UWBGL_WinForms_Lib1;

namespace XNA_SimulateGravity
{
    public class Model
    {
        private bool mAnimatingBall;
        private Ball mBall;
        private float mWorldWidth;
        private float mWorldHeight;
        private DateTime mPreviousUpdateTime;

        public Model()
        {
            mBall = new Ball();
            mWorldWidth = 0;
            mWorldHeight = 0;
            mAnimatingBall = false;
            mPreviousUpdateTime = DateTime.Now;
        }

        public void setWorldSize(float width, float height)
        {
            mWorldWidth = width;
            mWorldHeight = height;
        }

        public void setHeroBallAnimation(bool on)
        {
            mAnimatingBall = on;
        }

        public void createHeroBall(float worldX, float worldY)
        {
            mBall.SetCenter(new Vector3(worldX, worldY, 0));
            mBall.SetRadius(0);
            mBall.setVelocity(Vector3.Zero);
        }

        public void resizeHeroBall(float worldX, float worldY)
        {
            Vector3 center = mBall.GetCenter();
            float dx = worldX - center.X;
            float dy = worldY - center.Y;
            float radius = (float)Math.Sqrt(dx * dx + dy * dy);
            mBall.SetRadius(radius);
        }

        public void moveHeroBallTo(float worldX, float worldY)
        {
            mBall.SetCenter(new Vector3(worldX, worldY, 0));
        }

        public void updateSimulation()
        {
            TimeSpan elapsedTime = DateTime.Now - mPreviousUpdateTime;

            if (mAnimatingBall)
            {
                //update ball based on gravity
                float yAccel = -32f * (float)elapsedTime.TotalSeconds;
                Vector3 velocity = mBall.getVelocity();
                velocity.Y += yAccel;
                mBall.setVelocity(velocity);
               
                //update ball position
                Vector3 updateVector = mBall.getVelocity() * (float)elapsedTime.TotalSeconds;
                Vector3 center = mBall.GetCenter() + updateVector;
                mBall.SetCenter(center);

                //destroy balls that are outside of the world
                if(mBall.isOutsideBounds(mWorldWidth, mWorldHeight))
                {
                    mBall.setVelocity(Vector3.Zero);
                    mBall.SetRadius(0);
                    mAnimatingBall = false;
                }
            }
            mPreviousUpdateTime = DateTime.Now;
        }

        public void Draw()
        {
            mBall.Draw();
        }

        
    }
}
