using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib3
{
    public class UWB_XNARectangleGeometry
    {
        private float m_center_x;
        private float m_center_y;
        private float m_width;
        private float m_height;
        protected Color m_fillColor;
        protected bool m_fillMode;
        protected RasterizerState m_State;

        public UWB_XNARectangleGeometry()
        {
            m_center_x = 0;
            m_center_y = 0;
            m_width = 0;
            m_height = 0;
            m_fillColor = new Color(new Vector4(.4f, .4f, .4f, 1f));
            m_fillMode = true;

            m_State = new RasterizerState();
            m_State.FillMode = FillMode.Solid;
            m_State.CullMode = CullMode.None;
        }

        public void SetCenter(float x, float y)
        {
            m_center_x = x;
            m_center_y = y;
        }

        public void SetSize(float width, float height)
        {
            m_width = width;
            m_height = height;
        }

        public void setFillColor(Color fillColor)
        {
            m_fillColor = fillColor;
        }

        public Color getFillColor()
        {
            return m_fillColor;
        }

        public void setFillMode(bool on)
        {
            m_fillMode = on;
            if (on)
                m_State.FillMode = FillMode.Solid;
            else
                m_State.FillMode = FillMode.WireFrame;
        }

        public bool isFillModeOn()
        {
            return m_fillMode;
        }

        public void Draw()
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on Rectangle2D draw call");

            if (m_width > 0 || m_height > 0)
            {
                device.RasterizerState = m_State;

                VertexPositionColor[] v = new VertexPositionColor[4];

                for (int i = 0; i < 4; i++)
                {
                    v[i] = new VertexPositionColor();
                    v[i].Color = new Color(new Vector4(.4f, .4f, .4f, 1));
                }

                v[0].Position = new Vector3(m_center_x - m_width / 2.0f, m_center_y + m_height / 2.0f, 0);
                v[1].Position = new Vector3(m_center_x - m_width / 2.0f, m_center_y - m_height / 2.0f, 0);
                v[2].Position = new Vector3(m_center_x + m_width / 2.0f, m_center_y + m_height / 2.0f, 0);
                v[3].Position = new Vector3(m_center_x + m_width / 2.0f, m_center_y - m_height / 2.0f, 0);

                device.DrawUserPrimitives(PrimitiveType.TriangleStrip, v, 0, 2);
            }
        }
    }
}
