using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib4
{

	public class UWB_XNAGraphicsSystem
	{
		private BasicEffect m_BasicEffect;
		private GraphicsDevice m_XNA_GraphicsDevice;
        private RasterizerState m_DefaultState;

		public static UWB_XNAGraphicsSystem m_TheAPI = new UWB_XNAGraphicsSystem();

		private UWB_XNAGraphicsSystem()
		{
		}

		public bool CreateGraphicsContext(IntPtr hwnd, ref PresentationParameters pp)
		{
			//define presentation parameters
			pp = new PresentationParameters();
			pp.BackBufferFormat = SurfaceFormat.Color;
            pp.DeviceWindowHandle = hwnd;
            pp.IsFullScreen = false;
			pp.BackBufferHeight = Control.FromHandle(hwnd).Height;
			pp.BackBufferWidth = Control.FromHandle(hwnd).Width;

			if (m_XNA_GraphicsDevice != null)
				return false;
			//create an XNA graphics device
            m_XNA_GraphicsDevice = new GraphicsDevice(GraphicsAdapter.DefaultAdapter,
                GraphicsProfile.Reach, // allow support on as many devices as possible 
                pp);

            //check if graphics device was created
            Debug.Assert(m_XNA_GraphicsDevice != null, "XNA Graphics Device did not Initialize");

            //
            m_DefaultState = new RasterizerState();
            m_DefaultState.CullMode = CullMode.None;
            m_DefaultState.FillMode = FillMode.Solid;
            m_XNA_GraphicsDevice.RasterizerState = m_DefaultState;

            //check if graphics device was created
            Debug.Assert(m_XNA_GraphicsDevice != null, "XNA Graphics Device did not Initialize");

            m_BasicEffect = new BasicEffect(m_XNA_GraphicsDevice);

            //set the model matrix to identity
            m_BasicEffect.World = Matrix.Identity;
            m_BasicEffect.View = Matrix.Identity;
            m_BasicEffect.Projection = Matrix.Identity;

            return true;
		}

		public GraphicsDevice GetActiveDevice()
		{
			return m_XNA_GraphicsDevice;
		}

		public bool BeginScene(PresentationParameters pp)
		{
			if (m_XNA_GraphicsDevice == null)
				return false;
			m_XNA_GraphicsDevice.Reset(pp);
            m_XNA_GraphicsDevice.RasterizerState = m_DefaultState;
			m_XNA_GraphicsDevice.Clear(new Color(new Vector4(.8f, .8f, .8f, 0)));

			m_BasicEffect.VertexColorEnabled = true;

			m_BasicEffect.CurrentTechnique.Passes[0].Apply();

			return true;
		}

		public bool EndSceneAndShow(PresentationParameters pp)
		{
			if (m_XNA_GraphicsDevice == null)
				return false;

			m_XNA_GraphicsDevice.Present();

			return true;
		}

		public void setViewMatrix(Matrix matrix)
		{
			m_BasicEffect.World = matrix;
		}
	}
}
