using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib5;

namespace XNA_ObjectsAndAttributes
{
	public enum GraphicsObjectType
	{
		gotPoint,
		gotLine,
		gotCircle,
	};

	public class Model
	{
		private UWB_Primitive mCurrentObject;

		private UWB_PrimitiveCircle mCircle;
		private UWB_PrimitiveLine mLine;
		private UWB_PrimitivePoint mPoint;

		private UWB_Clock mStopWatch;
		private float mWorldWidth;
		private float mWorldHeight;

		private UWB_XNADrawHelper mDrawHelper;

		public Model()
		{
			mWorldWidth = 0;
			mWorldHeight = 0;

			mDrawHelper = new UWB_XNADrawHelper();

			mCircle = new UWB_PrimitiveCircle();
			mLine = new UWB_PrimitiveLine();
			mPoint = new UWB_PrimitivePoint();

			setCurrentTypeToPoint();

			mStopWatch = new UWB_Clock();
		}

		public void setWorldSize(float width, float height)
		{
			mWorldWidth = width;
			mWorldHeight = height;
		}

		public void getWorldSize(ref float width, ref float height)
		{
			width = mWorldWidth;
			height = mWorldHeight;
		}

		public void setCurrentTypeToPoint()
		{
			mCurrentObject = mPoint;
		}

		public void setCurrentTypeToLine()
		{
			mCurrentObject = mLine;
		}

		public void setCurrentTypeToCircle()
		{
			mCurrentObject = mCircle;
		}

		public void setObjectColor(Color color)
		{
			mCurrentObject.setFlatColor(color);
		}

		public void setShadingColor(Color color)
		{
			mCurrentObject.setShadingColor(color);
		}

		public void setShadeMode(eShadeMode shadeMode)
		{
			mCurrentObject.setShadeMode(shadeMode);
		}

		public void setPointSize(float size)
		{
			mCurrentObject.setPointSize(size);
		}

		public void setFillMode(eFillMode fillMode)
		{
			mCurrentObject.setFillMode(fillMode);
		}

		public eShadeMode getShadeMode()
		{
			return mCurrentObject.getShadeMode();
		}

		public eFillMode getFillMode()
		{
			return mCurrentObject.getFillMode();
		}

		public float getPointSize()
		{
			return mCurrentObject.getPointSize();
		}

		public void startDefineGraphicsObject(float worldX, float worldY)
		{
			mCurrentObject.mouseDownVertex(0, worldX, worldY);
			mCurrentObject.setVelocity(Vector3.Zero);
		}

		public void endDefineGraphicsObject(float worldX, float worldY)
		{
			mCurrentObject.mouseDownVertex(1, worldX, worldY);
		}

		public void moveGraphicsObject(float worldX, float worldY)
		{
			mCurrentObject.moveTo(worldX, worldY);
		}

		public bool isOutsideWorldBounds(Vector3 position)
		{
			if (position.X < 0 || position.X > mWorldWidth)
				return true;
			if (position.Y < 0 || position.Y > mWorldHeight)
				return true;

			return false;
		}

		public void updateSimulation()
		{
			Vector3 ptLoc = mPoint.getLocation();
			if(isOutsideWorldBounds(ptLoc))
				mPoint.setVelocity(-mPoint.getVelocity());

			Vector3 cirLoc = mCircle.getLocation();
			if(isOutsideWorldBounds(cirLoc))
				mCircle.setVelocity(-mCircle.getVelocity());

			Vector3 lnLoc = mLine.getLocation();
			if(isOutsideWorldBounds(lnLoc))
				mLine.setVelocity(-mLine.getVelocity());

			float elapsedSeconds = mStopWatch.getSecondsElapsed();
			mPoint.update(elapsedSeconds);
			mLine.update(elapsedSeconds);
			mCircle.update(elapsedSeconds);
		}

		public void drawModel()
		{
			eLevelofDetail lod = mDrawHelper.getLod();
			mCircle.draw(lod, mDrawHelper);
			mLine.draw(lod, mDrawHelper);
			mPoint.draw(lod, mDrawHelper);
		}

		public void placeGraphicsObjectInMotion()
		{
			float x = UWB_Utility.UWB_RandomNumber(-30, 30);
			float y = UWB_Utility.UWB_RandomNumber(-30, 30);
			Vector3 randomVelocity = new Vector3(x, y, 0);

			mCurrentObject.setVelocity(randomVelocity);
		}

		public void setLevelOfDetail(eLevelofDetail lod)
		{
			mDrawHelper.setLod(lod);
		}

		public eLevelofDetail getLevelOfDetail()
		{
			return mDrawHelper.getLod();
		}
	}
}

