using System;
using System.Collections.Generic;
using System.Text;
using UWBGL_XNA_Lib6;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

namespace UWBGL_XNA_Lib6
{
    public class UWB_XNADrawHelper : UWB_DrawHelper
    {
        public UWB_XNADrawHelper()
        {
        }

        public override bool drawPoint(Vector3 position)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawPoint call");

            // Draw points as rectangles
            VertexPositionColor[] v = new VertexPositionColor[4];
            float offset = mPointSize / 2f;

            for (int i = 0; i < 4; i++)
            {
                v[i] = new VertexPositionColor();
                v[i].Color = mColor1;
            }

            v[0].Position = new Vector3(position.X - offset, position.Y + offset, 0);
            v[1].Position = new Vector3(position.X - offset, position.Y - offset, 0);
            v[2].Position = new Vector3(position.X + offset, position.Y + offset, 0);
            v[3].Position = new Vector3(position.X + offset, position.Y - offset, 0);

            if (device.RasterizerState.FillMode != FillMode.Solid)
            {
                RasterizerState s = new RasterizerState();
                s.CullMode = CullMode.None;
                s.FillMode = FillMode.Solid;
                device.RasterizerState = s;
            }

            device.DrawUserPrimitives(PrimitiveType.TriangleStrip, v, 0, 2);
            return true;
        }

        public override bool drawLine(Vector3 start, Vector3 end)
        {
			GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
			Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");

            VertexPositionColor[] v = new VertexPositionColor[2];

			Color flatColor = mColor1;

			v[0].Position = start;
			v[0].Color = flatColor;
			v[1].Position = end;
			if (mShadeMode == eShadeMode.smFlat)
				v[1].Color = flatColor;
			else
				v[1].Color = mColor2;

			device.DrawUserPrimitives(PrimitiveType.LineList, v, 0, 1);

            return true;
        }

        public override bool drawCircle(Vector3 center, float radius)
        {
            GraphicsDevice device = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            Debug.Assert(device != null, "device not valid on DrawHelper drawCircle call");
            if (radius > 0)
            {
                int kNumTriangles = 40;
                if (mLod == eLevelofDetail.lodLow)
                    kNumTriangles = 10;
                else if (mLod == eLevelofDetail.lodMed)
                    kNumTriangles = 20;


                VertexPositionColor[] v = new VertexPositionColor[kNumTriangles * 3];

                float theta = (2.0f * (float)Math.PI) / kNumTriangles;

                v[0].Position = center;
                if (mShadeMode == eShadeMode.smGouraud)
                    v[0].Color = mColor2;
                else
                    v[0].Color = mColor1;

                float px = center.X + radius;
                float py = center.Y;

                for (int i = 0; i < kNumTriangles; i++)
                {
                    int offset = i * 3;
                    float x = center.X + (radius * (float)Math.Cos((i + 1) * theta));
                    float y = center.Y + (radius * (float)Math.Sin((i + 1) * theta));
                    v[offset].Position = v[0].Position;
                    v[offset].Color = v[0].Color;
                    v[offset + 1].Position = new Vector3(px, py, 0f);
                    v[offset + 1].Color = mColor1;
                    v[offset + 2].Position = new Vector3(x, y, 0f);
                    v[offset + 2].Color = mColor1;

                    px = x;
                    py = y;
                }

                device.DrawUserPrimitives(PrimitiveType.TriangleList, v, 0, kNumTriangles);
            }
            return true;
        }

        public override void setFillMode(eFillMode fillMode)
        {
            GraphicsDevice graphicsDevice = UWB_XNAGraphicsSystem.m_TheAPI.GetActiveDevice();
            bool needToSetMode = false;
            FillMode newMode = FillMode.Solid;
            switch (fillMode)
            {
                case eFillMode.fmWireframe:
                case eFillMode.fmPoint:
                    needToSetMode = (graphicsDevice.RasterizerState.FillMode != FillMode.WireFrame);
                    newMode = FillMode.WireFrame;
                    break;
                case eFillMode.fmSolid:
                    needToSetMode = (graphicsDevice.RasterizerState.FillMode != FillMode.Solid);
                    break;
            }
            RasterizerState s = new RasterizerState();
            s.CullMode = CullMode.None;
            s.FillMode = newMode;
            graphicsDevice.RasterizerState = s;
        }
    }
}