using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib8;

namespace XNA_TranslateSingleObject
{
	public class Model
	{
		private float mTranslationX;
		private float mTranslationY;

		private UWB_PrimitiveRectangle mRectangle;

		private UWB_Clock mStopWatch;

		private UWB_BoundingBox mWorldBounds;

		private UWB_XNADrawHelper mDrawHelper;

		public Model()
		{
			mRectangle = new UWB_PrimitiveRectangle();

			mWorldBounds = new UWB_BoundingBox();

			mDrawHelper = new UWB_XNADrawHelper();
			mDrawHelper.initializeModelTransform();

			mStopWatch = new UWB_Clock();

			mTranslationX = 0;
			mTranslationY = 0;

			mRectangle.setCorners(Vector3.Zero, new Vector3(2, 3, 0));
			mWorldBounds.setCorners(Vector3.Zero, new Vector3(12, 12, 0));
		}

		public void updateSimulation()
		{
			//nothing to do here
		}

		public void setWorldAspectRatio(float width, float height)
		{
			if (height > 0)
			{
				float worldHeight = 13;
				float ar = width / height;
				float worldWidth = ar * worldHeight;
				mWorldBounds.setCorners(Vector3.Zero, new Vector3(worldWidth, worldHeight, 0));
			}
		}

		public void getWorldBounds(ref float width, ref float height)
		{
			width = mWorldBounds.width();
			height = mWorldBounds.height();
		}

		public UWB_BoundingBox getWorldBounds()
		{
			return mWorldBounds;
		}

		public void setTranslation(float x, float y)
		{
			mTranslationX = x;
			mTranslationY = y;
		}

		public void drawGrid(UWB_DrawHelper drawHelper, UWB_BoundingBox bounds)
		{
			drawHelper.setShadeMode(eShadeMode.smFlat);
			drawHelper.setColor1(new Color(150, 150, 150));

			Vector3 minPoint = bounds.getMin();
			Vector3 maxPoint = bounds.getMax();

			float y = minPoint.Y;
			while (y < maxPoint.Y)
			{
				drawHelper.drawLine(new Vector3(minPoint.X, y, 0), new Vector3(maxPoint.X, y, 0));
				y += 1;
			}

			float x = minPoint.X;
			while (x < maxPoint.X)
			{
				drawHelper.drawLine(new Vector3(x, minPoint.Y, 0), new Vector3(x, maxPoint.Y, 0));
				x += 1;
			}
		}

		public void drawModel()
		{
			eLevelofDetail lod = mDrawHelper.getLod();
			drawGrid(mDrawHelper, mWorldBounds);

			Matrix translationMatrix = Matrix.Identity;
			translationMatrix.Translation = new Vector3(mTranslationX, mTranslationY, 0);

			UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(translationMatrix);

			mRectangle.draw(lod, mDrawHelper);

			UWB_XNAGraphicsSystem.m_TheAPI.setWorldMatrix(Matrix.Identity);
		}
	}
}

